/* 
  Charlieplexing
  16segment LED display rotate animation test
  New wiring
  by ikkei  2012.09.07
*/

#include <MsTimer2.h>
                // 1st digit pin assign
#define L01 1   // Anode ・・・・Common
#define L02 2   // G2
#define L03 3   // B
#define L04 4   // A2
#define L05 5   // J
#define L06 6   // I
#define L07 7   // A1
#define L08 8   // H
#define L09 9   // F
#define L10 12  // G1
#define L11 11  // E
#define L12 10  // D2
#define L13 A0  // M
#define L14 A1  // L
#define L15 A2  // D1
#define L16 A3  // K
#define L17 A4  // C
#define L18 A5  // decimal point

const byte LINE[ 6 ][ 18 ] = { 
{ L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L01, L02, L03, L04, L05, L18 },
{ L05, L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L01, L02, L03, L04, L18 },
{ L04, L05, L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L01, L02, L03, L18 },
{ L03, L04, L05, L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L01, L02, L18 },
{ L02, L03, L04, L05, L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L01, L18 },
{ L01, L02, L03, L04, L05, L06, L07, L08, L09, L10, L11, L12, L13, L14, L15, L16, L17, L18 },
};

const unsigned int FONT[]={ // G2,B,A2,J  I,A1,H,F  G1,E,D2,M  L,D1,K,C
0x0000, 0x0808, 0x4800, 0xC8AD, 0x2E2E, 0x1818, 0x1E0E, 0x3000,
0x1002, 0x0210, 0x1A1A, 0x8888, 0x0005, 0x8080, 0x8000, 0x1010,
0x7575, 0x0808, 0xE4E4, 0xE4A5, 0x8988, 0x25A6, 0xA5E5, 0x3410,
0xE5E5, 0xE5A5, 0x0808, 0x0810, 0x1002, 0x80A4, 0x0210, 0xE508,
0x64ED, 0xD011, 0xEC2D, 0x2564, 0x6C2D, 0xA5E4, 0xA5C0, 0xA565,
0xC1C1, 0x2C2C, 0x4065, 0x11C2, 0x0164, 0x5341, 0x4343, 0x6565,
0xE5C0, 0x6567, 0xE5C2, 0xA5A5, 0x2C08, 0x4165, 0x1150, 0x4153,
0x1212, 0x1208, 0x3434, 0x0560, 0x9288, 0x6005, 0x0300, 0x0024,
0x0200, 0x0CEC, 0x01E8, 0x00E0, 0x08EC, 0x00F0, 0xA888, 0x0DA8,
0x01C8, 0x0060, 0x0828, 0x180A, 0x080C, 0x80C9, 0x00C8, 0x00E8,
0x0DC0, 0x0D88, 0x00C0, 0x8006, 0x888C, 0x006C, 0x0050, 0x006D,
0x1212, 0xC805, 0x00B0, 0x288C, 0x0808, 0x8C28, 0x1300, 0xFFFF,
};

unsigned long segment[6] = { 0, 0, 0, 0, 0, 0 };

void rot_left( byte digit){
  const byte s1[8] = { 11, 14, 15, 1, 3, 6, 7, 9 };
  const byte s2[8] = { 12, 13, 16, 2, 4, 5, 8, 10 };
  byte a = bitRead(segment[ digit ], s1[0]);
  byte b = bitRead(segment[ digit ], s2[0]);
  for ( byte i=1; i < 8; i++){
    bitWrite(segment[ digit ], s1[i-1], bitRead(segment[ digit ], s1[i]));
    bitWrite(segment[ digit ], s2[i-1], bitRead(segment[ digit ], s2[i]));
  }
  bitWrite(segment[ digit ], s1[7], a);
  bitWrite(segment[ digit ], s2[7], b);
}

void alloff(void){
  for ( byte i=0; i < 18; i++){
    digitalWrite( LINE[ 0 ][ i ], LOW );
    pinMode( LINE[ 0 ][ i ], INPUT );
  }
}  

void LED_scan( void ){
  static byte digit = 0;
  // make dead time all output Off (Input)
  alloff();
  
  // anode common output High
  pinMode( LINE[ digit ][ 0 ], OUTPUT); 
  digitalWrite( LINE[ digit ][ 0 ], HIGH);
  
  // segment data output Low
  for ( byte i=0; i < 17; i++){
    if ( bitRead( segment[ digit ], i)){
      pinMode( LINE[ digit ][ 17-i ], OUTPUT);
    }
  }
  // next digit
  digit++;
  digit %= 6;
}

void setup(){
  alloff();
  MsTimer2::set( 2, LED_scan); // 2ms/digit
  MsTimer2::start();
  
  const char MSG[]="JH3KXM";
  for (byte i=0; i<6; i++){
    segment[i]=(unsigned long)FONT[ MSG[i] - ' ' ] << 1;
    if (i == 5){
      segment[i] |= 1; // decimal point
    }
  }
}


byte j=0;
void loop(){
  byte f;
  
  byte s[6][8];
  for (byte d=0; d<6; d++){
    for (byte k=0; k<8; k++){
      s[d][k] = bitRead(segment[ d ], k+1);
      bitWrite(segment[ d ], k+1, 0);
    }
  }
  
  for (byte i=0; i<80;i++){
    for (byte j=0; j<6;j++){
      rot_left(j);
    }
    delay(20);
  }
  
  for (byte d=0; d<6; d++){
    for (byte i=0; i<32;i++){
      for (byte j=d; j<6;j++){
        rot_left(j);
      }
      delay(20);
    }
    for (byte k=0; k<8; k++){
      bitWrite(segment[ d ], k+1, s[d][k]);
    }
  }
  
  delay(2000);
}


